import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from scipy.integrate import trapz

# Read the CO2 concentration values from the file
co2_ppm_file_path = "co2_ppm.txt"
try:
    co2_ppm_values = pd.read_csv(co2_ppm_file_path, header=None, names=['CO2_ppm'])
except FileNotFoundError:
    print(f"Error: File '{co2_ppm_file_path}' not found.")
    exit()
except pd.errors.EmptyDataError:
    print(f"Error: File '{co2_ppm_file_path}' is empty.")
    exit()

# Read the data from the CO2 bending mode file
co2_bend_file_path = "co2_bend.txt"
try:
    data = pd.read_csv(co2_bend_file_path, header=None, names=['Wavenumber', 'AbsorptionIntensity'])
except FileNotFoundError:
    print(f"Error: File '{co2_bend_file_path}' not found.")
    exit()
except pd.errors.EmptyDataError:
    print(f"Error: File '{co2_bend_file_path}' is empty.")
    exit()

# Initialize lists to store results for each CO2 concentration
trans_values = []
temp_surf_values = []

# Fixed temperature
temp = 288

# Loop through each CO2 concentration value
for co2_ppm in co2_ppm_values['CO2_ppm']:
    # Normalize CO2 concentration to ppm
    co2_ppm /= 1e6
    
    # Calculate cross-section number density
    xsec_num_density = 234972 * co2_ppm

    # Multiply each absorption intensity by 96.8 to obtain relative absorbance (Arel)
    data['RelativeAbsorbance'] = data['AbsorptionIntensity'] * xsec_num_density

    # Convert relative absorbance to flux transmittance using the provided formula
    data['FluxTransmittance'] = (np.exp(-(data['RelativeAbsorbance'] * 2.245)) + np.exp(-(data['RelativeAbsorbance'] * 1.13))) / 2

    # Generate wavenumbers from 0.01 to 2399.99 cm^-1
    wvn = np.arange(0.01, 2400, 0.01)

    # Calculate Planck distribution for the fixed temperature
    planck_distribution = (0.000000037211 * wvn**3) / (np.exp(1.435 * wvn / temp) - 1)

    # Multiply flux transmittance by Planck distribution at each wavenumber
    data['Result'] = data['FluxTransmittance'] * np.interp(data['Wavenumber'], wvn, planck_distribution)

# Save results to CSV files
result_df = pd.DataFrame({'Wavenumber': wvn, 'CO2 Planck': data['Result']})
result_df.to_csv('co2_ppm_trans_to_planck.csv', index=False)
print("Saved results to 'co2_ppm_trans_to_planck.csv'.")

result_df_planck = pd.DataFrame({'Wavenumber': wvn, 'Planck': planck_distribution})
result_df_planck.to_csv('planck_radiation_288.csv', index=False)
print("Saved results to 'planck_radiation_288.csv'.")

# Plot Planck distribution with CO2 Transmittance for a specific co2_ppm
plt.figure(figsize=(10, 6))
plt.plot(wvn, planck_distribution, label='Planck Distribution')
plt.xlabel('Wavenumber')
plt.ylabel('Intensity')
plt.title('Planck Distribution with CO2 Transmittance')
plt.legend()
plt.show()

# Select the index corresponding to the desired co2_ppm (e.g., 0 for the first row)
desired_co2_index = 0
desired_co2_ppm = co2_ppm_values['CO2_ppm'][desired_co2_index]

print("The CO2 ppm:", desired_co2_ppm)
# Plot CO2 Transmittance for a specific CO2 concentration
plt.figure(figsize=(10, 6))
plt.plot(data['Wavenumber'], data['Result'], label=f'CO2 Concentration: {desired_co2_ppm} ppm')
plt.xlabel('Wavenumber')
plt.ylabel('Intensity')
plt.title(f'CO2 Transmittance for CO2 Concentration: {desired_co2_ppm} ppm')
plt.legend()
plt.show()